import java.util.ArrayList;
import java.util.Random;


public class EAPRadiusHeader extends BasicPayloadObject {
	public byte  code;
    public byte  identifier;
    public short length;               // 2 bytes
	public byte[] authenticator = new byte[16]; 
	public ArrayList<EAPRadiusAttributes> attrList = null;
	protected boolean isLengthSet = false;


	public EAPRadiusHeader() {
		authenticator = new byte[16];
	}

	public EAPRadiusHeader(int code, int id) {
		this.code = (byte)code;
		this.identifier = (byte)id;
		authenticator = new byte[16];
		Random rnd;
		rnd = new Random(System.currentTimeMillis());
		rnd.nextBytes(authenticator);
	}

	public EAPRadiusHeader(int code, int id, byte[] auth) {                                                                                             this.code = (byte)code;
		this.code = (byte)code;
        this.identifier = (byte)id;
		if(auth != null) {
			authenticator = new byte[auth.length];
			System.arraycopy(auth, 0, authenticator, 0, auth.length);
		}
		else
			authenticator = new byte[16];
	}

	public void ReInit(int code, int id) {
		this.code = (byte)code;
		this.identifier = (byte)id;
		if(authenticator == null)
			authenticator = new byte[16];
		Random rnd;
		rnd = new Random(System.currentTimeMillis());
		rnd.nextBytes(authenticator);
		clearAttributesList();
	}

	public int getSize() {
		int res = 20;

		if(attrList != null) {
			int ext_len = 0;
			for (int j = 0; j < attrList.size(); j++) {
				ext_len += (attrList.get(j)).getSize();
			}
			res += ext_len;
		}
		return res;
	}

	public void setLength() {
		length = (short)getSize();
		isLengthSet = true;
	}

	public void setLength(int len) {
		length = (short)len;
		isLengthSet = true;
	}

	public void addAttribute(EAPRadiusAttributes attr) {
		if(attrList == null)
			attrList = new ArrayList<EAPRadiusAttributes>();
		attrList.add(attr);
	}

	public EAPRadiusAttributes getAttribute(int index) {
		EAPRadiusAttributes attr = null;
		try {
			attr = attrList.get(index);
		}
		catch (Exception ignored) {}
		
		return attr;
	}
	
	public EAPRadiusAttributes getAttributeByType(int type) {
		EAPRadiusAttributes attr = null;
		try {
			if(attrList != null)
				for(int i = 0; i<attrList.size(); i++) {
					if(attrList.get(i).attrType == type) {
						attr = attrList.get(i);
						break;
					}
				}
		}
		catch (Exception ignored) {}
		
		return attr;
	}
	
	public void removeAttribute(EAPRadiusAttributes attr) {
		attrList.remove(attr);
	}
	
	public void clearAttributesList() {
		if(attrList != null)
			attrList.clear();
	}
	
	public int getAttributesListLength() {
		int attr_len = 0;
		for (int j = 0; j < attrList.size(); j++) {
			attr_len += ((EAPRadiusAttributes)attrList.get(j)).getSize();
		}
		return attr_len;
	}

	public int parseNetMessage() {
		int res = 0;
		byte[] data = this.net_msg;
		if(data == null) 
			res = 1;
		else if(data.length < 20) 
			res = 2;
		else {
			this.code = data[0];
			this.identifier = data[1];
			this.length = (short)(data[2] & 0xFF);
			this.length <<= 8;
			this.length |= (data[3] & 0xFF);
			System.arraycopy(data, 4, this.authenticator, 0, 16);
			

			int curpos = 20;
			EAPRadiusAttributes newAttr = null;
			
			while(curpos < data.length - 1) {
				newAttr = new EAPRadiusAttributes(data,data[curpos++],data[curpos++]-2,curpos);
				this.addAttribute(newAttr);
				if(newAttr.value != null)
					curpos += newAttr.value.length;
			}
			if(curpos != data.length) {
				DebugInfo.printMessage(1,"EAPRadiusHeader::parseNetMessage: remain some bytes, length " + data.length + ", curpos " + curpos);
				res = 3;
			}
		}
		
		return res;
	}

	public int toByteArray() {
		int i = 0;
		if(isLengthSet == false) setLength();
		// create byte array
		byte[] ba = new byte[getSize()];
		
		ba[i++] = code;
		ba[i++] = identifier;
		ba[i++] = (byte)(length >>> 8);
		ba[i++] = (byte)length;
		if(authenticator != null) {
			System.arraycopy(authenticator, 0, ba, i, authenticator.length);
			i += authenticator.length;
		}
		//write extensions
		if(attrList != null) {
			byte[] attrArray;
			for (int j = 0; j < attrList.size(); j++) {
				if(attrList.get(j).net_msg == null)
					attrList.get(j).toByteArray();
				attrArray = attrList.get(j).net_msg;
				System.arraycopy(attrArray, 0, ba, i, attrArray.length);
				i += attrArray.length;
			}
		}

		this.net_msg = ba;
		return 0;
	}

	public void print() {
		DebugInfo.printMessage(1,"RADIUS packet:");
		DebugInfo.printMessage(1,"  code: " + RadiusCode.valueOf(this.code));
		DebugInfo.printMessage(1,"  identifier: " + this.identifier);
		DebugInfo.printMessage(1,"  length: " + this.length);
		DebugInfo.printDump(1, this.authenticator, "  authenticator:");
		DebugInfo.printMessage(1,"  Radius Attributes:");
		if(attrList != null) {
			for (int j = 0; j < attrList.size(); j++)
				attrList.get(j).print("  ");
		}
	}
}    
